/************************************************************************/
/*    author: LZX                                                       */
/*                                                                      */
/*                                                                      */
/************************************************************************/


#pragma once

#include "structinfo.h"

#define CLSID_PROXYMODULE L"{FAB59DC1-C950-4699-9647-085143BCD67C}"

#pragma pack(push, 1)

class IProxyReceptionCentre;
class IProxySettings;
class IProxyLog;
class IProxyTaskMgr;
class IProxyDataHandle;
class IProxyTester;

class IGlobalProxy
{
public:
	virtual ~IGlobalProxy(void){}

	virtual LPCSTR  GetLastErrorA() = 0;
	virtual LPCWSTR GetLastErrorW() = 0;

	virtual IProxyReceptionCentre* GetPRCInstance() = 0;
	virtual IProxySettings* GetSettingsInstance() = 0;
	virtual IProxyLog* GetLogInstance() = 0;

	virtual IProxyTester* CreateTester() = 0;

	virtual BOOL IsProxyEnabled() = 0;
	virtual BOOL EnableProxy() = 0;
	virtual BOOL DisableProxy() = 0;

	virtual INT AddRef() = 0;
	virtual INT Release() = 0;
};

class IProxyReceptionCentre
{
public:
	virtual ~IProxyReceptionCentre(void){}

	//ѯPRCͨѶĹܵ
	virtual BOOL GetPRCPipeName(LPSTR lpBuf, int bufsize) = 0;
	//ѯIProxyTaskMgr ʵָ
	virtual IProxyTaskMgr* GetPTMInstance(int type) = 0;
	//ѯIProxyDataHandle ʵָ
	virtual IProxyDataHandle* GetPDHInstance() = 0;
};

template<typename T>
class IInstanceList
{
public:
	IInstanceList(void)
	{
		m_pNext = NULL;
		m_pPrev = NULL;
	}
	virtual ~IInstanceList(void)
	{
		Detach();
	}

	virtual void AddInstance(T *pInstance)
	{
		if ( m_pNext && (m_pNext != pInstance) )
			m_pNext->AddInstance(pInstance);
		else
		{
			pInstance->m_pPrev = (T*)this;
			m_pNext = pInstance;
		}
	}

	virtual void Detach()
	{
		if (m_pNext)
			m_pNext->m_pPrev = m_pPrev;
		if (m_pPrev)
			m_pPrev->m_pNext = m_pNext;

		m_pPrev = m_pNext = NULL;
	}

	T *m_pPrev;
	T *m_pNext;
};


//////////////////////////////////////////////////////////////////////////
// IProxySettings
// ̳вظ࣬ GetGlobalProxyInstanceõʵָΪpIGlobalProxy
// ȻָyourPoint  pIGlobalProxy->GetSettingsInstance->AddInstance(yourPoint)
// 
// ֮ڲÿҪһ󶼻ᴥGetProxyInfo ʱĺԸpPRCCϢѡһϢlpPI
// 
//////////////////////////////////////////////////////////////////////////

class IProxySettings
	: public IInstanceList<IProxySettings>
{
public:
	virtual ~IProxySettings(void){}

	virtual BOOL GetProxySettings(LPProxySettingsInfo lpPSI) = 0;
	virtual BOOL GetProxyInfo(const LPPRCClient pPRCC, LPProxyInfo lpPI) = 0;

};



//////////////////////////////////////////////////////////////////////////
// IProxyLog
//
//////////////////////////////////////////////////////////////////////////

class IProxyLog
	: public IInstanceList<IProxyLog>
{
public:
	virtual ~IProxyLog(void){}

	virtual void LogText(LPCSTR lpText) = 0;
	virtual void LogNewProxyTask(const LPPRCClient lpC) = 0;
	virtual void OnNewProcess(LPHookNewProcessInfo lphnpi) = 0;

};


//////////////////////////////////////////////////////////////////////////
// IProxyTaskMgr
//
//////////////////////////////////////////////////////////////////////////

class IProxyTaskMgr
	: public IInstanceList<IProxyTaskMgr>
{
public:
	virtual ~IProxyTaskMgr(void){}

	virtual void OnAddTask(const LPPRCClient lpC, const LPProxyInfo lpPI) = 0;
	virtual void OnDelTask(const LPPRCClient lpC) = 0;
	virtual void EnumTask(IProxyTaskMgr *pCallBack) = 0;
	virtual void OnEachTask(const LPPRCClient lpC, const LPProxyInfo lpPI) = 0;

};


//////////////////////////////////////////////////////////////////////////
// IProxyDataHandle
//
//////////////////////////////////////////////////////////////////////////

class IProxyDataHandle
	: public IInstanceList<IProxyDataHandle>
{
public:
	virtual ~IProxyDataHandle(void){}

	virtual void OnConnect(LPProxyPacketInfo lpppi) = 0;
	virtual void OnEachPacket(LPProxyPacketInfo lpppi) = 0;
	virtual void OnClose(LPProxyPacketInfo lpppi) = 0;
	virtual void OnLayerCallback(LPProxyPacketInfo lpppi, int nType, int nCode, WPARAM wParam, LPARAM lParam) = 0;
};


//////////////////////////////////////////////////////////////////////////
// IProxyTesterCallback, IProxyTester
//
// pIGlobalProxyForTesting = GetGlobalProxyInstance();
//
// pTester = pIGlobalProxyForTesting->CreateTester; //
//
// pTester->Start(......);
//
// //ȡԻOnProxyTesterCallbackʱ ͷŶ
// pTester->Release();
//
// //ÿGetGlobalProxyInstanceĵ Ӧһ ReleaseGlobalProxyInstance 
// ReleaseGlobalProxyInstance();
//////////////////////////////////////////////////////////////////////////

class IProxyTesterCallback
{
protected:

//HIWORD(nErrorCode);
#define LAYERCALLBACK_STATECHANGE 0
#define LAYERCALLBACK_LAYERSPECIFIC 1

	enum LayerState
	{
		notsock,
		unconnected,
		connecting,
		listening,
		connected,
		closed,
		aborted
	};

#define PROXYERROR_NOERROR			0
#define PROXYERROR_NOCONN			1 //Can't connect to proxy server, use GetLastError for more information
#define PROXYERROR_REQUESTFAILED	2 //Request failed, can't send data
#define PROXYERROR_AUTHREQUIRED		3 //Authentication required
#define PROXYERROR_AUTHTYPEUNKNOWN	4 //Authtype unknown or not supported
#define PROXYERROR_AUTHFAILED		5 //Authentication failed
#define PROXYERROR_AUTHNOLOGON		6
#define PROXYERROR_CANTRESOLVEHOST	7

public:
	virtual ~IProxyTesterCallback(void){}

	//nErrorCode = MAKEWPARAM(nCode, nType)
	virtual void OnProxyTesterCallback(IProxyTester *pTester, int nErrorCode, WPARAM wParam, LPARAM lParam) = 0;
};


class IProxyTester
{
public:
	virtual ~IProxyTester(void){}

	virtual void Release() = 0;
	virtual BOOL Start(IProxyTesterCallback *pCallback, const LPPRCClient lpPRCClient, const LPProxyInfo lpPI) = 0;
	virtual void Stop() = 0;

};


////////////////////////////////////////
// һIGlobalProxyָ롣
// 
// ע⣺һGetGlobalProxyInstanceĵöӦһReleaseGlobalProxyInstance, ڲݼǷͷ
//
///////////////////////////////////////

IGlobalProxy* WINAPI GetGlobalProxyInstance();
BOOL WINAPI ReleaseGlobalProxyInstance();


////////////////////////////////////////
// ҪĽ̵API
//
///////////////////////////////////////


BOOL WINAPI gp_HookWinsock(LPCSTR lpszPRCPipeName);
BOOL WINAPI gp_UnhookWinsock();


#pragma pack(pop)